import { useContext } from "react";
import { TuitioContext, TuitioDispatchContext } from "./contexts";
import { TuitioClient, invalidate } from "@flare/tuitio-client";
import type { TuitioAuthenticationResult } from "@flare/tuitio-client";

export type TuitioClientHookOptions = {
  onLoginSuccess: (result: TuitioAuthenticationResult, userName: string) => void;
  onLoginFailed: (result: TuitioAuthenticationResult, userName: string) => void;
  onLoginError: (error: any) => void;
};

const useTuitioClient = (options: TuitioClientHookOptions) => {
  const state = useContext(TuitioContext);
  const dispatchActions = useContext(TuitioDispatchContext);

  const { onLoginSuccess, onLoginFailed, onLoginError } = options;

  const login = async (userName: string, password: string) => {
    try {
      const connector = new TuitioClient(state.configuration.tuitioUrl);
      const response = await connector.authenticate(userName, password);
      if (response.token) {
        dispatchActions.onLoginSuccess(response.token, userName);
        onLoginSuccess && onLoginSuccess(response, userName);
      } else {
        onLoginFailed && onLoginFailed(response, userName);
      }
      return response;
    } catch (error) {
      onLoginError && onLoginError(error);
      throw error;
    }
  };

  const logout = (): void => {
    invalidate();
    dispatchActions.onLogout();
  };

  return { login, logout };
};

const useTuitioUser = () => {
  const state = useContext(TuitioContext);
  const userName = state.userName;
  const lastLoginDate = state.token?.validFrom;
  return { userName, lastLoginDate };
};

const useTuitioToken = () => {
  const state = useContext(TuitioContext);
  const token = state.token;

  const validate = (): boolean => {
    const token = state.token;
    if (!token) {
      return false;
    }

    const valid = new Date(token.validUntil) >= new Date();
    return valid;
  };

  const valid: boolean = validate();

  return { token, validate, valid };
};

export { useTuitioClient, useTuitioUser, useTuitioToken };
